require 'thread'
require 'logger'
require 'glottis/handlers/console_input_handler'
require 'glottis/handlers/console_output_handler'
require 'glottis/handlers/remote_input_handler'
require 'glottis/handlers/remote_output_handler'

module Glottis
  # Client class, used to interact with the host specified.
  class Client
    def self.logger
      @logger ||= Logger.new(STDOUT)
    end

    def initialize(host, port, opts = {})
      @host = host
      @port = port
      @outgoing = Queue.new
      @incoming = Queue.new
      Client.logger.level = opts[:verbose] ? Logger::DEBUG : Logger::WARN
    end

    def run
      Client.logger.info('starting...')
      Thread.abort_on_exception = true

      @handlers = [
        Handlers::ConsoleInputHandler.new(@outgoing),
        Handlers::ConsoleOutputHandler.new(@incoming),
        Handlers::RemoteInputHandler.new(@incoming, @host, @port),
        Handlers::RemoteOutputHandler.new(@outgoing, @host, @port)
      ]

      @handlers.each(&:join)
    rescue Errno::ECONNREFUSED => conn_ref
      Client.logger.error("failed to connect: #{conn_ref}")
    rescue StandardError => ex
      Client.logger.warn("shutting down: #{ex}")
    ensure
      @handlers.each(&:cleanup)
      exit(0)
    end # run
  end # Client
end
