require 'logger'
require 'uri'
require 'net/http'
require 'json'

module Glottis
  module Handlers
    # This class manages a TCP connection with a valyx server.
    class RemoteInputHandler < Thread
      PROTOCOL = 'http'.freeze
      STREAM_DELIMITER = "\0".freeze
      READ_TIMEOUT = 3600 # this needs to be high
      REMOTE_PATHS = {
        get_message_stream: '/api/messages/stream'
      }.freeze

      def initialize(incoming, host, port)
        @incoming = incoming
        @host = host
        @port = port

        setup_http

        super do
          @http.start
          read_loop
        end
      end

      def cleanup
        @http.finish if @http.started?
      end

      private

      def setup_http
        @http = Net::HTTP.new(@host, @port)
        @http.open_timeout = 1
        @http.read_timeout = READ_TIMEOUT
      end

      def read_loop
        Client.logger.info('reading stream...')
        @http.get(REMOTE_PATHS.fetch(:get_message_stream)) do |chunk|
          Client.logger.debug("got chunk of length: #{chunk.length}")
          chunk.split(STREAM_DELIMITER).each do |msg_data|
            @incoming.push(JSON.parse(msg_data))
          end # each
        end # get
      end # read_loop
    end # RemoteInputHandler
  end # Handlers
end # Glottis
